#________________________________________________________________________________
#DESCRIPTION
#   Sun elevation routine using Vallado's algorithms
#________________________________________________________________________________
#USAGE
#   test_getSunElevation()
#________________________________________________________________________________
#REFERENCES
#    [1] Vallado, D. (2001) Fundamentals of Astrodynamics and Applications.
#    [2] Seidelmann, K. (1992) Explanatory Supplement to the Astronomical
#    Almanac. California University Science Books
#________________________________________________________________________________
#ISSUES
#    
#________________________________________________________________________________
#REVISIONS
#    2010/3/24  Creation
#________________________________________________________________________________
#REPORT BUGS TO
#   francoisroyer __at__ gmail.com or froyer __at__ cls.fr
#

library('chron')

################################################################################
# Support functions
################################################################################

deg2rad <- function(d) {
    return(d*pi/180.0)
}

rad2deg <- function(d) {
    return(d*180.0/pi)
}

lat2latgd <- function(thislatd) {
    eo=0.081819221456
    thislat = deg2rad(thislatd)
    return( rad2deg(asin(sin(thislat)/sqrt(1.0-(eo*acos(thislat))^2 ))) )
}

#For some reason, no norm() function in R's standard library!
norm <- function(a) {
 sqrt(t(a)%*%a)
}

datetime2julian <- function(y,m,d,hr,mn,sc) {
    as.double( julian(y,d,m , origin = c(2000,1,1)) ) + 2451544.5 + as.double( (hr*3600. + mn*60. +sc)/(24.*3600.) )
}

################################################################################
# Core functions
################################################################################

# Julian day as number of decimal days since Jan 1st, 4713 BC, 12:00:00
getSunElevation <- function(jd,latd,lond) {
    hell=0.0             ## Height (altitude) above the ellipsoid
    Ro=6378.1363       ## Mean equatorial radius of earth in kms (p 138 in [1])
    eo=0.081819221456  ## Eccentricity of earth
    ##deg2rad = pi/180.0
    lon=deg2rad(lond)   ## Convert to radians
    lat=deg2rad(latd)
    ## Geocentric to geodetic latitude
    latgd=asin(sin(lat)/sqrt(1-(eo*cos(lat))^2 ))
    halfpi=pi/2.0
    au = 149597870.0      ## Astronomical Unit (km)

    ## Observer location in geocentric IJK coordinates (Eq 3-14 p150 in [1])
    Co=Ro/sqrt(1-(eo*sin(latgd))^2 ) ## Radius of curvature in the meridian
    So=(1-eo^2)*Ro/sqrt(1-(eo*sin(latgd))^2 )
    rsite=array( c( (Co+hell)*cos(latgd)*cos(lon),
        (Co+hell)*cos(latgd)*sin(lon),
        (So+hell)*sin(latgd) ) )
    ## Sun position vector IJK - p264 in [1]
    TUT1=(jd-2451545.0)/36525.0 ## Number of julian centuries since epoch J2000
    lM=280.4606184+36000.77005361*TUT1
    M=357.5277233+35999.05034*TUT1
    lecl=lM+1.914666471*sin(deg2rad(M))+0.019994643*sin(2*deg2rad(M))
    eps=23.439291-0.0130042*TUT1   ## Obliquity of the ecliptic
    r=1.000140612-0.016708617*cos(deg2rad(M))-0.000139589*cos(2*deg2rad(M))

    rsun=au*r*array( c(cos(deg2rad(lecl)),
        cos(deg2rad(eps))*sin(deg2rad(lecl)),
        sin(deg2rad(eps))*sin(deg2rad(lecl))) ) #.transpose()

    ## Conversion to topocentric horizon coordinates IJK2SEZ - p254 in [1]
    GMST=67310.54841+(3600.0*876600.0+8640184.812866)*TUT1+0.093104*(TUT1^2)-(6.2e-6)*(TUT1^3)
    ## Greenwich mean sidereal time Eq 3-45 in [1] 
    GMST=sign(GMST)*(abs(GMST)%%86400.0)/240.0

    if (GMST< (-180.0)) { GMST=GMST+360.0 }
    LST=GMST+lond                ## Local sidereal time (deg)
    tLST=deg2rad(LST)  ## Local sidereal time (rads)

    ## Eq 3-26 p173 in [1]
    IJK2SEZ=t( array(c( sin(latgd)*cos(tLST),sin(latgd)*sin(tLST),-cos(latgd),
        -sin(tLST),cos(tLST),0.,
        cos(latgd)*cos(tLST),cos(lat)*sin(tLST),sin(latgd) ), dim = c(3,3)) )
    rhoIJK=rsun-rsite
    rhoSEZ= IJK2SEZ %*% rhoIJK
    ## Elevation and azimuth - RAZEL p254 in [1]
    rho = norm(rhoSEZ)

    return( rad2deg(asin(rhoSEZ[3]/rho)) )
}


################################################################################
# Test suite
################################################################################

test_getSunElevation <- function() {
    latd = 45.
    lond = 0.
    jd0 = datetime2julian(2000,1,1,12,0,0)
    jdays =  seq(jd0,jd0+10,by=0.01)
    N = length(jdays)
    el = array(0,N)
    #print(getSunElevation(jd0,latd,lond))
    i=1
    for (j in jdays) {
        el[i] = getSunElevation(j,latd,lond)
        i=i+1
    }
    plot(jdays,el)
}


